/*****************************************************************************
* FILENAME
*   main.c
*
* DESCRIPTION
*   Initializes EDMA, McBSP0, McBSP1, and aic23 codecs for interrupt-driven
*   block processing (allows overlapping blocks).
*
*   V1.0  Oct.02, 2002,  C.Feldbauer <feldbauer@tugraz.at>
*   V2.0  Aug.25, 2005,  C.Feldbauer (updated for C6713 DSK, uses CSL now)
*
*****************************************************************************/


#define CHIP_6713 1   // needed for Chip Support Library
#include <csl.h>      // Chip Support Library
#include <csl_edma.h>
#include <csl_irq.h>
#include <csl_mcbsp.h>

#include "dsk6713.h"
#include "dsk6713_aic23.h" // board, mcbsp, and codec init

#include "blocks.h"   // definition of block size, block overlap, ...



// more prototypes
void edma_init(); // see below...



// global variables (to be used in the ISR)...
int dma_index;
short in[NCHAN*NBLOCKS*HOPSZ], out[NCHAN*NBLOCKS*HOPSZ]; // actual DMA circular buffers

#if DATSZ != 2
  #error WRONG BUFFER TYPE FOR DMA.
#endif


//******************************* M A I N ( ) ********************************
int main()
{ 
  int i;
  DSK6713_AIC23_CodecHandle hCodec;
  DSK6713_AIC23_Config codec_conf = DSK6713_AIC23_CONF_32K; // Codec configuration
  
  
  CSL_init();
  IRQ_resetAll(); // disable ALL interrupts
  
  DSK6713_init(); // initialize the board
  
  hCodec = DSK6713_AIC23_openCodec(0, &codec_conf); // init mcbsp and codec
  
  
  for( i=0; i<NCHAN*NBLOCKS*HOPSZ; i++)  // clear all dma buffers
  {
    in[i]=0;
    out[i]=0;
  }  
    
  init(); // interface for additional initialization
  
  
  edma_init();
        
  // enable interrupts  
  IRQ_enable( IRQ_EVT_EDMAINT );
  IRQ_nmiEnable();
  IRQ_globalEnable();
  
  
  while(!DSK6713_AIC23_write( hCodec, 0)); // write a dummy sample to codec to enable new events
  while(!DSK6713_AIC23_read( hCodec, (Uint32 *)&i)); // read a sample from codec to enable new events
  // does this change the left-right order?
  
  while(1); // endless loop -> processing in ISR
}




// set up edma channel: events 14/15 (=mcbsp1 transmit/receive)...
void edma_init()
{
  int i, pcm_out, pcm_in;
  EDMA_Handle hEdmaX, hEdmaR, hEdmaXlnk[NBLOCKS], hEdmaRlnk[NBLOCKS];
  
  EDMA_resetAll();
  
  // allocate reload PaRAMs
  for(i=0;i<NBLOCKS;i++)
  {
    hEdmaRlnk[i] = EDMA_allocTable(-1);
    hEdmaXlnk[i] = EDMA_allocTable(-1);
  }  
  
  /* set up edma for mcbsp1 transmitter...
     OPT = 0x49000002;
     { FS=0
       LINK=1   ... reload PaRAMs
       TCC=0000 
       TCINT=0  ... no interrupt generation here (see receiver)
       DUM=00
       2DD=0
       SUM=01   ... source address increment
       2DS=0    ... no 2D transfer
       ESIZE=01 ... 16bit half-word
       PRI=010  ... low priority
     }
     
     set up edma for mcbsp1 receiver
     OPT = 0x48380002;
     { FS=0
       LINK=1   ... reload PaRAMs
       TCC=1000 ... CIPR[8] will be set (chosen arbitrarily)
       TCINT=1  ... interrupt generation
       DUM=01   ... destination address increment
       2DD=0    ... no 2D transfer
       SUM=00
       2DS=0
       ESIZE=01 ... 16bit half-word
       PRI=010  ... low priority
     }  
  */
  
  pcm_in  = (int)&in;
  pcm_out = (int)&out;
  
  // mcbsp1 transmit event
  hEdmaX = EDMA_open( EDMA_CHA_XEVT1, EDMA_OPEN_ENABLE);
  EDMA_configArgs( hEdmaX, 0x49000002, pcm_out, NCHAN*HOPSZ, MCBSP_getXmtAddr( DSK6713_AIC23_DATAHANDLE), 0, EDMA_getTableAddress(hEdmaXlnk[1]));
  //               handle, OPT,        SRC,     CNT,         DST,                                         ?, reload_PaRAM_address
    
  // mcbsp1 receive event
  hEdmaR = EDMA_open( EDMA_CHA_REVT1, EDMA_OPEN_ENABLE);
  EDMA_configArgs( hEdmaR, 0x48380002, MCBSP_getRcvAddr( DSK6713_AIC23_DATAHANDLE), NCHAN*HOPSZ, pcm_in, 0, EDMA_getTableAddress(hEdmaRlnk[1]));
  //               handle, OPT,        SRC,                                         CNT,         DST,    ?, reload_PaRAM_address
    
  // setup and link reload PaRAMs
  for(i=0;i<NBLOCKS;i++)
  {
    // transmit
    EDMA_configArgs( hEdmaXlnk[i], 0x49000002, pcm_out+i*NCHAN*HOPSZ*DATSZ, NCHAN*HOPSZ, MCBSP_getXmtAddr( DSK6713_AIC23_DATAHANDLE), 0, EDMA_getTableAddress(hEdmaXlnk[(i+1)%NBLOCKS]));
    //               handle,       OPT,        SRC,                         CNT,         DST,                                         ?, reload_PaRAM_address
    
    // receive
    EDMA_configArgs( hEdmaRlnk[i], 0x48380002, MCBSP_getRcvAddr( DSK6713_AIC23_DATAHANDLE), NCHAN*HOPSZ, pcm_in+i*NCHAN*HOPSZ*DATSZ, 0, EDMA_getTableAddress(hEdmaRlnk[(i+1)%NBLOCKS]));
    //               handle,       OPT,        SRC,                                         CNT,         DST,                        ?, reload_PaRAM_address
  }
  
  EDMA_intEnable( 8);
  
  dma_index = 1;
  /* Note, when the ISR is called for the first time, 'dma_index==1' tells us the
     index of the EDMA frame processed in the background. The newest data is at 'dma_index==0'.
  */
}
